"""Run a test against mock data."""

from pathlib import Path
from unittest.mock import MagicMock, patch

import pytest

from hcb_soap_client.hcb_soap_client import HcbSoapClient
from tests.test_data.const import ACCOUNT_ID

_empty = ""


@patch("hcb_soap_client.hcb_soap_client.aiohttp.ClientSession")
@pytest.mark.asyncio
async def test_account_response(mock: MagicMock) -> None:
    """Tests the account response."""
    session = MagicMock()
    session.post.return_value.__aenter__.return_value.status = 200
    session.post.return_value.__aenter__.return_value.text.return_value = _read_file(
        "S1157.xml"
    )
    mock.return_value.__aenter__.return_value = session
    client = HcbSoapClient()
    response = await client.get_parent_info(_empty, _empty, _empty)
    if response.account_id != ACCOUNT_ID:
        msg = "Account id does not match."
        raise ValueError(msg)
    expected_students = 2
    if len(response.students) != expected_students:
        msg = "Student count does not match."
        raise ValueError(msg)


@patch("hcb_soap_client.hcb_soap_client.aiohttp.ClientSession")
@pytest.mark.asyncio
async def test_stop_response(mock: MagicMock) -> None:
    """Tests the account response."""
    session = MagicMock()
    session.post.return_value.__aenter__.return_value.status = 200
    session.post.return_value.__aenter__.return_value.text.return_value = _read_file(
        "s1158_AM.xml"
    )
    mock.return_value.__aenter__.return_value = session
    client = HcbSoapClient()
    response = await client.get_stop_info(_empty, _empty, _empty, _empty)
    if response.vehicle_location.address == "":
        msg = "address not found."
        raise ValueError(msg)
    expected_stops = 2
    if len(response.student_stops) != expected_stops:
        msg = "Student count does not match."
        raise ValueError(msg)


def _read_file(file_name: str) -> str:
    """Read a text file."""
    with Path(f"tests/test_data/{file_name}").open() as file:
        return file.read()



"""Class for the account info."""

from dataclasses import dataclass
from datetime import time

import xmltodict

from . import from_list, from_str, from_time


@dataclass
class Student:
    """Info for the student."""

    student_id: str
    first_name: str
    last_name: str

    @staticmethod
    def from_dict(xml_dict: dict) -> "Student":
        """Create a new instance of from a dictionary."""
        student_id = from_str(xml_dict.get("@EntityID"))
        first_name = from_str(xml_dict.get("@FirstName"))
        last_name = from_str(xml_dict.get("@LastName"))
        return Student(student_id, first_name, last_name)


@dataclass
class TimeOfDay:
    """The time of day list."""

    id: str
    name: str
    begin_time: time
    end_time: time

    @staticmethod
    def from_dict(xml_dict: dict) -> "TimeOfDay":
        """Create a new instance of from a dictionary."""
        _id = from_str(xml_dict.get("@ID"))
        name = from_str(xml_dict.get("@Name"))
        begin_time = from_time(xml_dict.get("@BeginTime"))
        end_time = from_time(xml_dict.get("@BeginTime"))
        return TimeOfDay(_id, name, begin_time, end_time)


@dataclass
class AccountResponse:
    """Parent account info."""

    account_id: str
    students: list[Student]
    times: list[TimeOfDay]

    @staticmethod
    def from_text(response_text: str) -> "AccountResponse":
        """Create a new instance of from text."""
        data = xmltodict.parse(response_text, force_list={"Student"})
        data = data["s:Envelope"]["s:Body"]["s1157Response"]
        data = data["s1157Result"]["SynoviaApi"]["ParentLogin"]
        account_id = data["Account"]["@ID"]
        students = from_list(Student.from_dict, data["LinkedStudents"].get("Student"))
        times = from_list(TimeOfDay.from_dict, data["TimeOfDays"].get("TimeOfDay"))
        return AccountResponse(account_id, students, times)



"""Init file for the soap client."""

from collections.abc import Callable
from datetime import datetime, time
from typing import Any, TypeVar

from dateutil import parser

T = TypeVar("T")


def from_str(x: Any) -> str:
    """Get a string from an object."""
    return x


def from_datetime(x: Any) -> datetime:
    """Get a datetime from an object."""
    return parser.parse(x)


def from_time(x: Any) -> time:
    """Get a datetime from an object."""
    return from_datetime(x).time()


def from_float(x: Any) -> float:
    """Get a float from an object."""
    return float(from_str(x))


def from_int(x: Any) -> int:
    """Get an int from an object."""
    return int(from_str(x))


def from_bool(x: Any) -> bool:
    """Get a bool from an object."""
    string = from_str(x)
    return string.startswith("Y")


def from_list(f: Callable[[Any], T], x: Any) -> list[T]:
    """Get a list."""
    return [f(y) for y in x]



"""Define a stop object."""

from datetime import datetime, time

import xmltodict
from attr import dataclass

from . import (
    from_bool,
    from_datetime,
    from_float,
    from_int,
    from_list,
    from_str,
    from_time,
)


@dataclass
class StudentStop:
    """Define a student stop."""

    name: str
    latitude: float
    longitude: float
    start_time: time
    stop_type: str
    substitute_vehicle_name: str
    vehicle_name: str
    stop_id: str
    arrival_time: time
    time_of_day_id: str
    vehicle_id: str
    esn: str
    tier_start_time: datetime
    bus_visibility_start_offset: int

    @staticmethod
    def from_dict(xml_dict: dict) -> "StudentStop":
        """Create a new instance of from a dictionary."""
        name = from_str(xml_dict.get("@Name"))
        latitude = from_float(xml_dict.get("@Latitude"))
        longitude = from_float(xml_dict.get("@Longitude"))
        start_time = from_time(xml_dict.get("@StartTime"))
        stop_type = from_str(xml_dict.get("@StopType"))
        substitute_vehicle_name = from_str(xml_dict.get("@SubstituteVehicleName"))
        vehicle_name = from_str(xml_dict.get("@VehicleName"))
        stop_id = from_str(xml_dict.get("@StopId"))
        arrival_time = from_time(xml_dict.get("@ArrivalTime"))
        time_of_day_id = from_str(xml_dict.get("@TimeOfDayId"))
        vehicle_id = from_str(xml_dict.get("@VehicleId"))
        esn = from_str(xml_dict.get("@Esn"))
        tier_start_time = from_datetime(xml_dict.get("@TierStartTime"))
        bus_visibility_start_offset = from_int(
            xml_dict.get("@BusVisibilityStartOffset")
        )
        return StudentStop(
            name,
            latitude,
            longitude,
            start_time,
            stop_type,
            substitute_vehicle_name,
            vehicle_name,
            stop_id,
            arrival_time,
            time_of_day_id,
            vehicle_id,
            esn,
            tier_start_time,
            bus_visibility_start_offset,
        )


class VehicleLocation:
    """Define a student vehicle location."""

    name: str
    latitude: float
    longitude: float
    log_time: datetime
    ignition: bool
    latent: bool
    time_zone_offset: int
    heading: str
    speed: int
    address: str
    message_code: int
    display_on_map: bool

    def __init__(  # noqa: PLR0913
        self,
        name: str,
        latitude: float,
        longitude: float,
        log_time: datetime,
        ignition: bool,  # noqa: FBT001
        latent: bool,  # noqa: FBT001
        time_zone_offset: int,
        heading: str,
        speed: int,
        address: str,
        message_code: int,
        display_on_map: bool,  # noqa: FBT001
    ) -> None:
        """Create a new instance."""
        self.name = name
        self.latitude = latitude
        self.longitude = longitude
        self.log_time = log_time
        self.ignition = ignition
        self.latent = latent
        self.time_zone_offset = time_zone_offset
        self.heading = heading
        self.speed = speed
        self.address = address
        self.message_code = message_code
        self.display_on_map = display_on_map

    @staticmethod
    def from_dict(xml_dict: dict) -> "VehicleLocation":
        """Create a new instance from a dictionary."""
        if xml_dict is None:
            return None

        name = from_str(xml_dict.get("@Name"))
        latitude = from_float(xml_dict.get("@Latitude"))
        longitude = from_float(xml_dict.get("@Longitude"))
        log_time = from_datetime(xml_dict.get("@LogTime"))
        ignition = from_bool(xml_dict.get("@Ignition"))
        latent = from_bool(xml_dict.get("@Latent"))
        time_zone_offset = int(from_str(xml_dict.get("@TimeZoneOffset")))
        heading = from_str(xml_dict.get("@Heading"))
        speed = from_int(from_str(xml_dict.get("@Speed")))
        address = from_str(xml_dict.get("@Address"))
        message_code = from_int(from_str(xml_dict.get("@MessageCode")))
        display_on_map = from_bool(xml_dict.get("@DisplayOnMap"))
        return VehicleLocation(
            name,
            latitude,
            longitude,
            log_time,
            ignition,
            latent,
            time_zone_offset,
            heading,
            speed,
            address,
            message_code,
            display_on_map,
        )


@dataclass
class StopResponse:
    """Define a stop object."""

    vehicle_location: VehicleLocation
    student_stops: list[StudentStop]

    @staticmethod
    def from_text(response_text: str) -> "StopResponse":
        """Create a new instance of from text."""
        data = xmltodict.parse(response_text)
        data = data["s:Envelope"]["s:Body"]["s1158Response"]["s1158Result"][
            "SynoviaApi"
        ]["GetStudentStopsAndScans"]["GetStudentStops"]
        vehicle_location = VehicleLocation.from_dict(data.get("VehicleLocation"))
        stops = []
        if data["StudentStops"] is not None:
            student_stops = data["StudentStops"].get("StudentStop")
            stops = from_list(StudentStop.from_dict, student_stops)
        return StopResponse(vehicle_location, stops)



"""Define the setup."""

from pathlib import Path

import setuptools

with Path("README.md").open() as fh:
    long_description = fh.read()

setuptools.setup(
    name="hcb_soap_client",
    version="0.0.7",
    author="Patrick Cartwright",
    author_email="pcartwright1981@gmail.com",
    description="Soap client for Here Comes the Bus",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pcartwright81/hcb_soap_client",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
